'use strict';

var logger = require('./logger')(module);
var Promise = require('bluebird');
var enums = require('./enums');
var queueGetJob = require('./queue-get-job');
var jobAddLog = require('./job-add-log');

module.exports = function jobUpdate(job) {
  logger('jobUpdate:  [' + job.id + ']');

  var log = jobAddLog.createLogObject(job, null, enums.message.jobUpdated, enums.log.information);

  return Promise.resolve().then(function () {
    return queueGetJob(job.q, job.id);
  }).then(function (oldJobs) {
    log.data = oldJobs[0].getCleanCopy();
    delete log.data.log;
    job.log.push(log);
    return job.getCleanCopy();
  }).then(function (cleanJob) {
    return job.q.r.db(job.q.db).table(job.q.name).get(job.id).update(cleanJob, { returnChanges: false }).run();
  }).then(function (updateResult) {
    logger('updateResult', updateResult);
    logger('Event: updated [' + job.id + ']');
    job.q.emit(enums.status.updated, job.id);
    return true;
  });
};