'use strict';

var logger = require('./logger')(module);
var Promise = require('bluebird');
var enums = require('./enums');
var is = require('./is');
var queueProcess = require('./queue-process');
var queueState = require('./queue-state');

module.exports.pause = function interruptionPause(q, source) {
  logger('pause', source);
  q._paused = true;
  var makeGlobal = is.true(source);
  var eventGlobal = makeGlobal || source === enums.state.global;
  return q.ready().then(function () {
    if (makeGlobal) {
      return queueState(q, enums.status.paused);
    }
    return;
  }).then(function () {
    return new Promise(function (resolve, reject) {
      logger('Event: pausing [' + q.id + ']');
      q.emit(enums.status.pausing, eventGlobal, q.id);
      if (q.running < 1) {
        return resolve();
      }
      var intId = setInterval(function pausing() {
        logger('Pausing, waiting on running jobs: [' + q.running + ']');
        if (q.running < 1) {
          clearInterval(intId);
          resolve();
        }
      }, 400);
    });
  }).then(function () {
    logger('Event: paused [global:' + eventGlobal + '] [' + q.id + ']');
    q.emit(enums.status.paused, eventGlobal, q.id);
    return true;
  });
};

module.exports.resume = function interruptionResume(q, source) {
  logger('resume', source);
  q._paused = false;
  var makeGlobal = is.true(source);
  var eventGlobal = makeGlobal || source === enums.state.global;
  return q.ready().then(function () {
    if (makeGlobal) {
      return queueState(q, enums.status.active);
    }
    return;
  }).then(function () {
    queueProcess.restart(q);
    logger('Event: resumed [global:' + eventGlobal + '] [' + q.id + ']');
    q.emit(enums.status.resumed, eventGlobal, q.id);
    return true;
  });
};